/**
 *  ContourMap.java -- 
 *  (Plotter.java Ȥ)
 *
 *  @version $Revision: 1.12 $, $Date: 2003/03/19 17:50:28 $
 */

import java.awt.*;
import javax.swing.*;
import java.io.*;

public class ContourMap extends JPanel {
    /** ؿ */
    public interface Function {
        abstract double f(double x, double y);
        abstract double fx(double x, double y);
        abstract double fy(double x, double y);
    }

    static final int WIDTH = 400,  HEIGHT = 400;
    static final double SCALE = 10;
    static final double EPS = 0.001;  // Ƹ $\varepsilon$

    public ContourMap() {
        setPreferredSize(new Dimension(WIDTH + 1, HEIGHT + 1));
    }

    private double square(double x) {  return x * x;  }

    private double x, y, fx, fy, grad2;

    /** Newtonˡ (x,y,fx,fy,grad2 ͤ򥻥åȤ) */
    private void newton(Function func, double fc, double x, double y) {
        double t;
        do {
            double f = func.f(x, y);
            fx = func.fx(x, y);  fy = func.fy(x, y);
            grad2 = square(fx) + square(fy);
            if (grad2 < 1e-10) throw new Error("grad2 ޤ");
            t = (fc - f) / grad2;
            x += fx * t;  y += fy * t;
        } while (t * t * grad2 > EPS * EPS);
        this.x = x;  this.y = y;
    }

    /**  */
    public void draw(Function func, double fc,
                     double x0, double y0, double step) {
        Graphics2D g = (Graphics2D)getGraphics();
        g.translate(WIDTH / 2, HEIGHT / 2);  g.scale(1, -1);  // 濴
        Plotter pt = new Plotter(g);

        x = x0;  y = y0;
        for (int i = 0; ; i++) {
            newton(func, fc, x, y);  // x,y,fx,fy,grad2 ͤ
            if (Math.abs(x) + Math.abs(y) > 1e10) return;
            if (i == 0) {
                   pt.moveTo(x * SCALE, y * SCALE);  x0 = x;  y0 = y;
            } else pt.drawTo(x * SCALE, y * SCALE);
            if (i >= 2 && square(x - x0) + square(y - y0) < square(step))
                break;
            double t = step / Math.sqrt(grad2);
            x += fy * t;  y -= fx * t;
        }
        pt.drawTo(x0 * SCALE, y0 * SCALE);
    }

    public static void main(String[] args) throws IOException {
        JFrame frame = new JFrame("");
        ContourMap cm = new ContourMap();
        frame.getContentPane().add(cm);
        frame.pack();               // Ŭڤ礭
        frame.setDefaultCloseOperation(frame.EXIT_ON_CLOSE);
        frame.setResizable(false);  // ѹԲ
        frame.setVisible(true);     // ɽ

        BufferedReader input =
            new BufferedReader(new InputStreamReader(System.in));
        ContourMap.Function func = new ContourMap.Function() {
                public double f(double x, double y) {
                    return x * x + 4 * y * y;  // $f(x, y) = x^2 + 4y^2$
                }
                public double fx(double x, double y) {
                    return 2 * x;  // $x$ ʬ: $f_x(x, y) = 2x$
                }
                public double fy(double x, double y) {
                    return 8 * y;  // $y$ ʬ: $f_y(x, y) = 8y$
                }
            };

        for ( ; ; ) {
            System.out.print("f(x, y) = ");    // δؿ
            double fc = Double.parseDouble(input.readLine());
            System.out.print("initial x = ");  // $x$ ν
            double x = Double.parseDouble(input.readLine());
            System.out.print("initial y = ");  // $y$ ν
            double y = Double.parseDouble(input.readLine());
            System.out.print("step = ");       // ƥåץ
            double step = Double.parseDouble(input.readLine());
            cm.draw(func, fc, x, y, step);
        }
    }
}
