/**
 *  ʬ
 *
 *  @version $Revision: 1.1 $, $Date: 2003/04/05 10:44:10 $
 */

import java.io.*;

class FactorAnalysis {
    static StreamTokenizer input;
    static void factor(int nfac, double[][] r, double[][] q,
                       double[] lambda, double[] work) throws IOException {
        int iter = 0, maxiter = 0, m = r.length;
        NumberFormatter fmtRMS = new NumberFormatter(".3g");
        for ( ; ; ) {
            if (++iter > maxiter) {
                System.out.print("֤ (0:֤λ) ? ");
                while (input.nextToken() != input.TT_NUMBER);
                if (input.nval <= 0) break;
                maxiter += (int)input.nval;
            }
            for (int j = 0; j < m; j++) for (int k = 0; k < m; k++)
                q[j][k] = r[j][k];
            if (QRDiagonalizer.eigen(q, lambda, work) == false) {
                System.err.println("«ޤ");  System.exit(1);
            }
            double s = RVector.innerProduct(nfac, lambda, lambda);
            System.out.println(iter + ": гʬ RMS  " + 
                               fmtRMS.toString(Math.sqrt(s / (m * (m - 1)))));
            for (int j = 0; j < m; j++) {
                double sum = 0;
                for (int k = 0; k < nfac; k++)
                    sum += lambda[k] * q[k][j] * q[k][j];
                r[j][j] = sum;
            }
        }
        double t = 0;
        for (int k = 0; k < m; k++) t += lambda[k];
        NumberFormatter fmtf = new NumberFormatter("3d"),
                        fmte = new NumberFormatter("8.4f"),
                        fmtp = new NumberFormatter("5.1f");
        System.out.println("    ͭ      ѡ");
        double s = 0;
        for (int k = 0; k < m; k++) {
            if (k < nfac)
                 System.out.print(" " + fmtf.toString(k + 1) + " ");
            else System.out.print("(" + fmtf.toString(k + 1) + ")");
            double percent = 100 * lambda[k] / t;  s += percent;
            System.out.println(" " + fmte.toString(lambda[k]) + 
                               "  " + fmtp.toString(percent) + 
                               "  " + fmtp.toString(s));
        }
        System.out.println("  " + fmte.toString(t) + 
                           "  " + fmtp.toString(s));
        for (int k = 0; k < nfac; k++)
            work[k] = Math.sqrt(Math.abs(lambda[k]));
        fmtf = new NumberFormatter("4d");
        fmte = new NumberFormatter("6.3f");
        fmtp = new NumberFormatter("7.3f");
        System.out.println("ѿ     ");
        for (int j = 0; j < m; j++) {
            System.out.print(fmtf.toString(j + 1) + "  " + 
                             fmte.toString(r[j][j]) + " ");
            for (int k = 0; k < nfac; k++) {
                q[k][j] *= work[k];
                if (k < 9) System.out.print(fmtp.toString(q[k][j]));
            }
            System.out.println();
        }
    }

    public static void main(String[] args) throws IOException {
        if (args.length != 1) {
            System.err.println("ˡ: java FactorAnalysis filename");
            return;
        }
        StatData data = null;
        int n = 0, m = 0;
        double[][] r = null, q = null;
        double[] mean = null, lambda = null, work = null;
        try {
            data = new StatData(args[0]); // throw RuntimeException
            n = data.numberOfRows(); m = data.numberOfColumns();
            r = new double[m][m]; q = new double[m][m];
            mean = new double[m]; lambda = new double[m];
            work = new double[m];
            for (int i = 0; i < n; i++) for (int j = 0; j < m; j++) {
                double t = data.getNum(); // throw RuntimeException
                work[j] = t - mean[j];
                mean[j] += work[j] / (i + 1);
                for (int k = 0; k <= j; k++)
                    r[j][k] += i * work[j] * work[k] / (i + 1);
            }
        } finally {
            data.close();
        }
        for (int j = 0; j < m; j++) {
            work[j] = Math.sqrt(r[j][j]); r[j][j] = 1;
            for (int k = 0; k < j; k++) {
                r[j][k] /= work[j] * work[k]; r[k][j] = r[j][k];
            }
        }
        double t = 1 / Math.sqrt(n - 1.0);
        NumberFormatter fmtd = new NumberFormatter("4d"),
                        fmtg = new NumberFormatter(" -12.5g");
        System.out.println("ѿ    ʿ      ɸк");
        for (int j = 0; j < m; j++)
            System.out.println(fmtd.toString(j + 1) + "  " + 
                               fmtg.toString(mean[j]) + "  " + 
                               fmtg.toString(t * work[j]));
        fmtg = new NumberFormatter("8.4f");
        System.out.println("ط");
        for (int j = 0; j < m; j++) {
            for (int k = 0; k <= j; k++)
                System.out.print(fmtg.toString(r[j][k]) + " ");
            System.out.println();
        }
        input = new StreamTokenizer(new InputStreamReader(System.in));
        for ( ; ; ) {
            System.out.print("\n̰Ҥο (0:¹Խλ) ? ");
            while (input.nextToken() != input.TT_NUMBER);
            int nfac = (int)input.nval;
            if (nfac > m) nfac = m;
            if (nfac < 1) break;
            factor(nfac, r, q, lambda, work);
        }
    }
}
