/**
 * Huffman.java -- Huffman沽
 *
 * @version $Revision: 1.18 $, $Date: 2003/03/22 02:37:57 $
 * BitInputStream, BitOutputStreamȤ
 */

import java.io.*;

public class Huffman {
    static final int N = 256;   // ե٥åȤΥ (ʸ = 0..N-1)
    private int heapSize, avail;
    private int[] heap = new int[2 * N - 1]; // ͥԤѥҡ
    private int[] parent = new int[2 * N - 1]; // HuffmanڤΥǡ¤
    private int[] left = new int[2 * N - 1]; // 
    private int[] right = new int[2 * N - 1]; // 
    private int[] freq = new int[2 * N - 1]; // ʸνи

    /*
     * ͥԤ
     */
    private void downHeap(int i) {
        int j, k = heap[i];

        while ((j = 2 * i) <= heapSize) {
            if (j < heapSize && freq[heap[j]] > freq[heap[j + 1]])
                j++;
            if (freq[k] <= freq[heap[j]]) break;
            heap[i] = heap[j];  i = j;
        }
        heap[i] = k;
    }

    /*
     * Huffmanڤν
     */
    private void writeTree(BitOutputStream out, int i) throws IOException {
        if (i < N) {            // 
            out.putBit(false);
            out.putBits(8, i);  // ʸΤ
        } else {                // 
            out.putBit(true);
            writeTree(out, left[i]); // λ
            writeTree(out, right[i]); // λ
        }
    }

    /*
     * 沽
     */
    public void encode(String infile, String outfile) throws IOException {
        int i, j, k;
        BufferedInputStream in = new BufferedInputStream(new FileInputStream(infile));
        BitOutputStream out = new BitOutputStream(new BufferedOutputStream(new FileOutputStream(outfile)));
        boolean[] codeBit = new boolean[N]; // 

        for (i = 0; i < N; i++) freq[i] = 0; // ٤ν
        while ((i = in.read()) >= 0) freq[i]++; // ٿ
        heap[1] = 0;            // Ĺ0Υե
        heapSize = 0;
        for (i = 0; i < N; i++)
            if (freq[i] != 0) heap[++heapSize] = i; // ͥԤϿ
        for (i = heapSize / 2; i >= 1; i--) downHeap(i); // ҡ׺
        k = heap[1];            // ʲΥ롼פ1¹Ԥʤ
        avail = N;              // ʲΥ롼פHuffmanڤ
        while (heapSize > 1) {  // 2İʾĤ꤬
            i = heap[1];        // ǾǤФ
            heap[1] = heap[heapSize--];  downHeap(1); // ҡ׺ƹ
            j = heap[1];        // ˺ǾǤФ
            k = avail++;        // 
            freq[k] = freq[i] + freq[j];    // ٤
            heap[1] = k;    downHeap(1);    // ԤϿ
            parent[i] = k;  parent[j] = -k; // ڤ
            left[k] = i;    right[k] = j;   // 
        }
        parent[k] = 0;            // 
        out.putBits(out.MAX_BITS, freq[k]); // ϥեΥХȿ
        writeTree(out, k);        // ڤ
        int tableSize = out.outCount(); // ɽ礭
        int inCount = 0;
        in.close();              // ǽ
        in = new BufferedInputStream(new FileInputStream(infile));
        while ((j = in.read()) >= 0) {
            k = 0;
            while ((j = parent[j]) != 0)
                if (j > 0) codeBit[k++] = false;
                else {     codeBit[k++] = true;  j = -j;  }
            while (--k >= 0) out.putBit(codeBit[k]);
            if ((++inCount & 1023) == 0) System.err.print('.'); // 
        }
        System.err.println("\nIn : " + inCount +  " bytes");  // 
        System.err.println("Out: " + out.outCount() +
                           " bytes (table: " + tableSize + " bytes)");
        if (inCount != 0) {     // 
            long cr = (1000L * out.outCount() + inCount / 2) / inCount;
            System.err.println("Out/In: " + (cr / 1000) + "." + (cr % 1000));
        }
        in.close();  out.close(); // ϥե򥯥
    }

    /*
     * Huffmanڤɤ
     */
    private int readTree(BitInputStream in) throws IOException {
        if (in.getBit()) { // bit=1: դǤʤ
            int i;
            if ((i = avail++) >= 2 * N - 1) {
                System.err.println("ɽְäƤޤ");
                System.exit(-1);
            }
            left [i] = readTree(in); // λޤɤ
            right[i] = readTree(in); // λޤɤ
            return i;           // ֤
        } else return in.getBits(8); // ʸ
    }

    /*
     * 
     */
    public void decode(String infile, String outfile) throws IOException {
        BitInputStream in = new BitInputStream(new BufferedInputStream(new FileInputStream(infile)));
        BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(outfile));
        int size = in.getBits(in.MAX_BITS); // ΥХȿ
        avail = N;
        int root = readTree(in); // ڤɤ
        for (int k = 0; k < size; k++) { // ʸ
            int j = root;  // 
            while (j >= N)
                if (in.getBit()) j = right[j];  else j = left[j];
            out.write(j);
            if ((k & 1023) == 0) System.err.print('.');
        }
        System.err.println("\nOut: " + size + " bytes"); // 椷Хȿ
        in.close();  out.close(); // ϥե򥯥
    }

    /*
     * ᥤ
     */
    public static void main(String[] args) throws IOException {
        if (args.length != 3 || !(args[0].equals("e") || args[0].equals("d"))) {
            System.err.println("ˡ: java Huffman [e|d] infile outfile");
            System.exit(-1);
        }
        Huffman huf = new Huffman();
        if (args[0].equals("e")) huf.encode(args[1], args[2]); // 沽
        else                     huf.decode(args[1], args[2]); // 
    }
}
