/**
 *  륻̡ĥ (mt19937ar 2002/1/26)
 *
 *  @version $Revision: 1.7 $, $Date: 2003/03/30 09:23:45 $
 */

import java.text.DecimalFormat;

public class MersenneTwister extends java.util.Random {
    static final int N = 624;
    static final int M = 397;
    static final int UPPER_MASK = 0x80000000;  // 1ӥå
    static final int LOWER_MASK = 0x7FFFFFFF;  // 31ӥå
    static final int MATRIX_A   = 0x9908B0DF;
    int x[] = new int[N];  // ֤򵭲ʥ󥰥Хåե
    int p, q, r;  // 󥰥ХåեΥǥå

    public MersenneTwister() {  setSeed(System.currentTimeMillis());  }
    public MersenneTwister(long seed) {  setSeed(seed);  }
    public MersenneTwister(int[] seeds) {  setSeed(seeds);  }

    synchronized public void setSeed(long seed) {
        if (x == null) return;  //  super() θƤӽФǤϲ⤷ʤ
        x[0] = (int)seed;
        for (int i = 1; i < N; i++)
            x[i] = 1812433253 * (x[i - 1] ^ (x[i - 1] >>> 30)) + i;
        p = 0;  q = 1;  r = M;
    }

    synchronized public void setSeed(int[] seeds) {
        setSeed(19650218);
        int i = 1,  j = 0;
        for (int k = 0; k < Math.max(N, seeds.length); k++) {
            x[i] ^= (x[i - 1] ^ (x[i - 1] >>> 30)) * 1664525;
            x[i] += seeds[j] + j;
            if (++i >= N) {  x[0] = x[N - 1];  i = 1;  }
            if (++j >= seeds.length) j = 0;
        }
        for (int k = 0; k < N - 1; k++) {
            x[i] ^= (x[i - 1] ^ (x[i - 1] >>> 30)) * 1566083941;
            x[i] -= i;
            if (++i >= N) {  x[0] = x[N - 1];  i = 1;  }
        }
        x[0] = 0x80000000;
    }

    synchronized protected int next(int bits) {
        int y = (x[p] & UPPER_MASK) | (x[q] & LOWER_MASK);
        y = x[p] = x[r] ^ (y >>> 1) ^ ((y & 1) * MATRIX_A);
        if (++p == N) p = 0;  // 󥰥Хåեź +1
        if (++q == N) q = 0;
        if (++r == N) r = 0;

        y ^= (y >>> 11);  // ¿ʬۤɤĴ
        y ^= (y  <<  7) & 0x9D2C5680;
        y ^= (y  << 15) & 0xEFC60000;
        y ^= (y >>> 18);
        return (y >>> (32 - bits));
    }

    // http://www.math.keio.ac.jp/~matumoto/emt.html
    // ΥɤƱ̤Ϥ
    public double genrand_real2() {
        long x = next(32);
        if (x < 0) x += 1L << 32;
        return x * (1.0 / 4294967296.0);
    }

    public static void main(String[] args) {
        int[] seeds = {0x123, 0x234, 0x345, 0x456};
        MersenneTwister random = new MersenneTwister(seeds);
        System.out.println("1000 outputs of genrand_int32()");
        for (int i = 0; i < 1000; i++) {
            long x = random.nextInt();
            if (x < 0) x += 1L << 32;
            String s = "         " + x + " ";
            System.out.print(s.substring(s.length() - 11));
            if (i % 5 == 4) System.out.println();
        }
        System.out.println();
        System.out.println("1000 outputs of genrand_real2()");
        DecimalFormat df = new DecimalFormat("0.00000000 ");
        for (int i = 0; i < 1000; i++) {
            System.out.print(df.format(random.genrand_real2()));
            if (i % 5 == 4) System.out.println();
        }
    }
}
