/**
 *  Newton1.java -- Newton (˥塼ȥ) ˡ
 *
 *  @version $Revision: 1.12 $, $Date: 2003/04/05 15:10:00 $
 */

import java.io.*;
import java.util.*;
import java.text.DecimalFormat;

/**
 * <code><i>f</i>(<i>x</i>) = 0</code>β
 * Newton (˥塼ȥ) ˡȤäƵޤ
 * <br>
 * @see Cardano
 * @see Newton1Test
 * @see Newton2
 */
public class Newton1 {
    /** ᤿ؿ f(x) */
    public interface Function {
        public double of(double x);       // ؿ f(x)
        public double primeOf(double x);  // Ƴؿ f'(x)
    }

    static final double EPS = 1.0e-14;
    static final DecimalFormat DF2  = new DecimalFormat(" 0.0E00;-0.0E00");
    static final DecimalFormat DF15 = new
        DecimalFormat(" 0.000000000000000E00;-0.000000000000000E00");

    /**  $x$ Ϳ $f(x) = 0$ β֤ */
    public static double solve(Function f, double x) {
        double xPrev;
        do {
            double fx = f.of(x), fp;
            System.out.println("  x = " + DF15.format(x) +
                               "  f(x) = " + DF2.format(fx));
            if ((fp = f.primeOf(x)) == 0) fp = 1;  // ˤ餹
            xPrev = x;  x -= fx / fp;
        } while (Math.abs(x - xPrev) > EPS);
        return x;
    }

    /**
     *  a, b, c, d ǶڤäϤȣ<br>
     * <code>ax<sup>3</sup> + bx<sup>2</sup> + cx + d = 0</code><br>
     * μ¿ɽޤ
     * @see Cardano
     */
    public static void main(String[] args) throws IOException {
        /** f(x) = x^3 + bx^2 + c^x + d */
        class CubicFunction implements Newton1.Function {
            private double b, c, d;
            public CubicFunction(double a, double b, double c, double d) {
                this(b / a, c / a, d / a);
            }
            public CubicFunction(double b, double c, double d) {
                this.b = b;  this.c = c;  this.d = d;
            }
            public double of(double x) { // ᤿ؿ $f(x)$
                return ((b + x) * x + c) * x + d;
            }
            public double primeOf(double x) { // $f(x)$ Ƴؿ
                return (2 * b + 3 * x) * x + c;
            }
        }

        System.out.println("ax^3+bx^2+cx+d=0򤭤ޤ.");
        System.out.println("a b c d ? ");
        BufferedReader in =
            new BufferedReader(new InputStreamReader(System.in));
        StringTokenizer st = new StringTokenizer(in.readLine());
        double a = Double.parseDouble(st.nextToken()),
               b = Double.parseDouble(st.nextToken()),
               c = Double.parseDouble(st.nextToken()),
               d = Double.parseDouble(st.nextToken());
        if (a == 0) {
            System.out.println("3ǤϤޤ");  System.exit(-1);
        }
        b /= a;  c /= a;  d /= a;
        a = b * b - 3 * c;
        CubicFunction cf = new CubicFunction(b, c, d);

        if (a > 0) {
            double mp = (-b + Math.sqrt(a))/3.0;    // ˾κɸ
            a = (2.0 / 3.0) * Math.sqrt(a);
            double x1 = Newton1.solve(cf, -a - b / 3);  // ¦顤˾ëϤ뤳ȤǤ
            System.out.println("x1 = " + x1);
            if (cf.of(mp) > 0) System.exit(0);      // ˾ͤ :Ƚ꤬ʤ 2 5 2 6 ʤɤǼ«ʤ
            double x2 = Newton1.solve(cf, a - b / 3);   // ¦
            if (x2 == x1) System.exit(0);
            System.out.println("x2 = " + x2);
            double x3 = Newton1.solve(cf, b / (-3));    // Ѷ
            System.out.println("x3 = " + x3);
        } else {
            double x1 = Newton1.solve(cf, 0);           // Ŭ
            System.out.println("x = " + x1);
        }
    }
}
