/**
* Squeeze.java -- LZˡ
* @version $Revision: 1.11 $, $Date: 2003/03/22 02:40:56 $
* BitOutputStream.java  BitInputStream.java Ȥ
*/

import java.io.*;

/**
* ưŪˡLZˡˤǡ沽ΥץΣĤǤ
* @see BitInputStream
* @see BitOutputStream
* @see Huffman
* @see Slide
*/

public class Squeeze {
    static final int N        =  256;    // ʸμ (ʸ = 0..N-1)
    static final int MAX_DICT = 4096;    // 񥵥 4096, 8192, ...
    static final int MAX_MATCH = 100;    // Ĺ
    static final int NIL =  MAX_DICT;    // ΡֹȤ¸ߤʤ
    private char character[] = new char[MAX_DICT];
    private int[] parent= new int[MAX_DICT], lChild = new int[MAX_DICT]; // , λ
    private int[] rSib  = new int[MAX_DICT], lSib   = new int[MAX_DICT]; // Τ礦
    private int[] newer = new int[MAX_DICT], older  = new int[MAX_DICT]; // Ԥݥ
    private int[] match = new int[MAX_MATCH];  // ʸ
    private int dictSize = N;           // ߤμ񥵥
    private int qIn = NIL, qOut = NIL;  // Ԥ, и
    private int bitLen = 1;   // ߤĹ
    private int bitMax = 2;   // 1 << bitLen

    private static void error(String msg) {
        System.err.println(msg);  System.exit(-1);
    }
    // Ρ p  LRU Ԥ󤫤鳰 (size > 1; p ϺǸǤʤ)
    private void dequeue(int p) {
        if (p == qOut) {  // Ƭξ
            qOut = newer[p];  older[qOut] = NIL;
        } else {
            int o = older[p], n = newer[p];
            newer[o] = n;  older[n] = o;
        }
    }

    // Ρ p Ԥ q θ (q  NIL ʤǽ)
    private void enqueue(int p, int q) {
        if (qIn == NIL) {  // Ԥ󤬶
            older[p] = newer[p] = NIL;  qIn = qOut = p;
        } else if (q == NIL) {  // Ԥκǽդ
            older[p] = NIL;  newer[p] = qOut;
            qOut = older[qOut] = p;
        } else if (q == qIn) {  // ԤκǸդ
            older[p] = qIn;  newer[p] = NIL;
            qIn = newer[qIn] = p;
        } else {  // Ԥ˳
            older[p] = q;
            newer[p] = newer[q];
            newer[q] = older[newer[p]] = p;
        }
    }

    // Ρ p ʸ c Ҥ֤ (ʤ NIL)
    private int child(int p, int c) {
        p = lChild[p];
        while (p != NIL && c != character[p]) p = rSib[p];
        return p;
    }

    // ƥΡ parp ʸ c ҤȤեΡ p 
    private void addLeaf(int parp, int p, int c) {
        character[p] = (char)c;
        parent[p] = parp;
        lChild[p] = lSib[p] = NIL;
        int q = lChild[parp];  rSib[p] = q;
        if (q != NIL) lSib[q] = p;
        lChild[parp] = p;
    }

    // եΡ p 
    private void deleteLeaf(int p) {
        int left = lSib[p], right = rSib[p];

        if (left != NIL) rSib[left] = right;
        else      lChild[parent[p]] = right;
        if (right != NIL) lSib[right] = left;
    }

    // ڤν
    private void initTree() {
        dictSize = N;  qIn = NIL;  qOut = NIL;  bitLen   = 1;  bitMax = 2; // ٻȤʤ
        for (int i = 0; i < N; i++) {
            character[i] = (char)i;
            parent[i] = lChild[i] = lSib[i] = rSib[i] = NIL;
        }
    }

    // ڤι
    private void update(int[] match, int curLen, int prevPtr, int prevLen) {
        if (prevPtr == NIL) return;

        for (int i = 0; i < curLen; i++) {
            if (++prevLen > MAX_MATCH) return;
            int c = match[i], p;
            if ((p = child(prevPtr, c)) == NIL) {
                if (dictSize < MAX_DICT) p = dictSize++;  /* dictSize < NIL */
                else {
                    if (prevPtr == qOut) return;
                    p = qOut;  dequeue(p);  deleteLeaf(p);
                }
                addLeaf(prevPtr, p, c);
                if (prevPtr < N) enqueue(p, qIn);
                else             enqueue(p, older[prevPtr]);
            }
            prevPtr = p;
        }
    }

    private void output(BitOutputStream out, int p) throws IOException {
        if (p < N) {
            out.putBit(false);  out.putBits(8, p);
        } else {
            while ((dictSize - N) >= bitMax) {
                bitLen++;  bitMax <<= 1;
            }
            out.putBit(true);  out.putBits(bitLen, p - N);
        }
    }

    private int input(BitInputStream in) throws IOException {
        if ((dictSize - N) >= bitMax) {
            bitMax <<= 1;  bitLen++;
        }
        if (! in.getBit()) return in.getBits(8);
        return in.getBits(bitLen) + N;
    }
    /**
    *沽Ԥޤ
    *@param inFile ϥե̾
    *@param outFile ϡ沽˥ե̾
    */
    public void encode(String inFile, String outFile) throws IOException {
        // ϡϥȥ꡼򥪡ץ
        BufferedInputStream in = new BufferedInputStream(new FileInputStream(inFile));
        BitOutputStream out = new BitOutputStream(new BufferedOutputStream(new FileOutputStream(outFile)));
        int size = (int)(new File(inFile)).length();// ϥեΥХȿ
        out.putBits(out.MAX_BITS, size);          // եإå˥Хȿ񤭹

        int curPtr = NIL, curLen = 0, prevPtr, prevLen;
        int c = in.read();
        long inCount = 0, printCount = 0;

        initTree();
        while (c != -1) {
            prevPtr = curPtr;  prevLen = curLen;  curLen = 0;
            int  p = c, q = qIn;
            do {
                if (p >= N)
                    if (p == q) q = older[p];
                    else {  dequeue(p);  enqueue(p, q);  }
                match[curLen++] = c;  curPtr = p;
                c = in.read();  p = child(curPtr, c);
            } while (p != NIL);
            output(out, curPtr);
            update(match, curLen, prevPtr, prevLen);
            if ((inCount += curLen) > printCount) {
                System.err.print('.');   printCount += 1024;  // 
            }
        }
        System.out.println("\nIn : " + inCount +  " bytes");
        System.out.println("Out: " + out.outCount() +  " bytes");
        if (inCount != 0) {
            long cr = (1000L * out.outCount() + inCount / 2) / inCount;
            System.out.println("Out/In: " + (cr / 1000) + "." + (cr % 1000));
        }
        in.close();  out.close(); // ϥե򥯥
    }
    /**
    * Ԥޤ
    *@param inFile 沽ե̾
    *@param outFile ե̾
    */
    public void decode(String inFile, String outFile) throws IOException {
        BitInputStream in = new BitInputStream(new BufferedInputStream(new FileInputStream(inFile)));
        BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(outFile));
        int curPtr = NIL, curLen = 0, prevPtr, prevLen;
        int[] base = new int[MAX_MATCH];
        long size = in.getBits(in.MAX_BITS); // ΥХȿ
        long outCount = 0, printCount = 0;

        initTree();
        while (outCount < size) {
            int p = input(in);
            if (p >= dictSize) error("ϥ顼");
            prevPtr = curPtr;  prevLen = curLen;
            curPtr = p;  curLen = 0;
            while (p != NIL) {
                if (p >= N && p != qIn) {
                    dequeue(p);  enqueue(p, qIn);
                }
                curLen++;
                match[MAX_MATCH - curLen] = character[p];
                p = parent[p];
            }
            int bs = MAX_MATCH - curLen;
            System.arraycopy(match, bs, base, 0, match.length - bs);
            for (int i = 0; i < curLen ; i++) out.write(base[i]);
            update(base, curLen, prevPtr, prevLen);
            if ((outCount += curLen) > printCount) {
                System.err.print('.');  printCount += 1024;
            }
        }
        System.out.println("\nOut: " + outCount + " bytes"); // 椷Хȿ
        in.close();  out.close(); // ϥե򥯥
    }
    /**
    * ˡ<br>
    * <code>java Squeeze [e|d] infile outfile</code><br>
    * ץ e : 沽d : 
    */
// Huffman.java Ʊ
    public static void main(String[] args) throws IOException {
        if (args.length != 3 || !(args[0].equals("e") || args[0].equals("d"))) {
            System.err.println("ˡjava Squeeze [e|d] infile outfile");
            System.exit(-1);
        }
        Squeeze sqz = new Squeeze();
        if (args[0].equals("e")) sqz.encode(args[1], args[2]);  // 沽
        else                     sqz.decode(args[1], args[2]);  // 
    }
}
